using System;
using Syncfusion.Windows.Forms.Diagram;
using System.Collections.Specialized;
using System.Runtime.Serialization;
using System.Collections;
using System.Windows.Forms;
using System.Drawing;

namespace Syncfusion.Windows.Forms.Diagram.Samples.OrgChart
{
	/// <summary>
	/// Summary description for ConditionSysbol.
	/// </summary>
	/// 
	[Serializable()]
	public class clsAlternativeSymbol:tmpSymbol
	{
		public Rectangle rect = null;

		private FieldsEnum mTextField=FieldsEnum.Code;
		private System.Collections.Specialized.NameValueCollection mItems=new NameValueCollection();
	
		public enum FieldsEnum {Code=1,Description,CodeAndDescription,None};

		private const float DEFAULT_LABEL_WIDTH=80;
		private const float DEFAULT_LABEL_HEIGHT=20;
		private const string UNTITLED_LABEL="Untitled";

		//private const Color  ALTER_SYMBOL_COLOR=Color.Beige; 
		//private const Color  HIGHLIGHT_SYMBOL_COLOR=Color.BurlyWood; 
		//private const Color  HIGHLIGHT_SYMBOL_COLOR=Color.BurlyWood; 

		public clsAlternativeSymbol()
		{
			this.rect = new Rectangle(0, 0,DEFAULT_LABEL_WIDTH,DEFAULT_LABEL_HEIGHT);
			this.rect.Name="Rectangle";
			this.rect.FillStyle.Color = Color.Beige;
			this.AppendChild(this.rect);
			
		}
		protected clsAlternativeSymbol(SerializationInfo info, StreamingContext context) : base(info, context)
		{

		}

		protected override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			base.GetObjectData(info, context);
		}

		protected override void OnDeserialization(object sender)
		{
			base.OnDeserialization(sender);

			// Get references to the child objects
			this.rect = this.GetChildByName("Rectangle") as Syncfusion.Windows.Forms.Diagram.Rectangle;
		}
	
		
		public void Clear()
		{
			mItems.Clear();
			ClearLabels();
		}
		public void AddItem(string Code,string Description)
		{
			mItems.Add(Code,Description);
			// add the new item label
			
			
			Label lblItem = this.AddLabel("",BoxPosition.TopCenter);
			
			lblItem.Visible=false;
			lblItem.Text=Code;
			lblItem.FontStyle.Style = System.Drawing.FontStyle.Regular; //Bold;
			lblItem.FontStyle.Family="Tahoma";
			lblItem.FontStyle.Size=10F;

			lblItem.BackgroundStyle.Color=System.Drawing.Color.Transparent;
			lblItem.ReadOnly=true;
			lblItem.HorizontalAlignment=System.Drawing.StringAlignment.Center;
			lblItem.VerticalAlignment=System.Drawing.StringAlignment.Center;
			lblItem.WrapText=true;
			lblItem.Width=this.Width;
			lblItem.Height=DEFAULT_LABEL_HEIGHT;
			lblItem.Visible=false;
			ResizeLabels();
		}

		public void RemoveItem(string Code)
		{
			mItems.Remove(Code);
			for (int i=this.Labels.Count-1; i>=0; i--)
			{
				if (this.Labels[i].Text==Code)
				{
					this.Labels.RemoveAt(i);
					break;
				}
			}
			ResizeLabels();
		}
		public FieldsEnum TextField
		{
			set 
			{
				mTextField=value;
				ResizeLabels();
			}
			get {return mTextField;}

		}
		private void ClearLabels()
		{
			//Remove Lables in MySymbol
			for (int i=this.Labels.Count-1; i>=0; i--)
			{
				this.Labels.RemoveAt(i);
			}
			
		}
		private void SetLabelToMiddle()
		{
			int iTotal=this.Labels.Count;
			if (iTotal>0)
			{
				// width
				//
				
				for (int i=this.Labels.Count-1; i>=0; i--)
				{
					this.Labels[i].Width=this.Width;
				}
				if (this.Height>=DEFAULT_LABEL_HEIGHT*iTotal)
				{	
					// rectangle size is >= total labels' default size
					float fOffsetYBegin=(this.Height-DEFAULT_LABEL_HEIGHT*iTotal)/2;
					for (int i=0;i<this.Labels.Count;i++)
					{
						this.Labels[i].OffsetY=fOffsetYBegin+DEFAULT_LABEL_HEIGHT*i+DEFAULT_LABEL_HEIGHT/2;
					}
				}
				else
				{
					// rectangle size is < total labels' default size
					float fLabelHeight=this.Height/iTotal;
					for (int i=0;i<this.Labels.Count;i++)
					{
						this.Labels[i].Height=fLabelHeight;
						this.Labels[i].OffsetY=fLabelHeight*i+fLabelHeight/2;
					}
				}
				for (int i=this.Labels.Count-1; i>=0; i--)
				{
					this.Labels[i].Visible=true;
				}
				
			}
						
		}

		private void ResizeLabels()
		{
			int iTotal=this.Labels.Count;
			if (iTotal>0)
			{
				// width
				//
				
				for (int i=this.Labels.Count-1; i>=0; i--)
				{
					this.Labels[i].Width=this.Width;
				}
				if (this.Height<DEFAULT_LABEL_HEIGHT*iTotal) this.Height=DEFAULT_LABEL_HEIGHT*iTotal;
				float fOffsetYBegin=(this.Height-DEFAULT_LABEL_HEIGHT*iTotal)/2;
				for (int i=0;i<this.Labels.Count;i++)
				{
					this.Labels[i].OffsetY=fOffsetYBegin+DEFAULT_LABEL_HEIGHT*i+DEFAULT_LABEL_HEIGHT/2;
				}
				for (int i=this.Labels.Count-1; i>=0; i--)
				{
					this.Labels[i].Visible=true;
				}
				
			}
						
		}
		protected override void OnMouseEnter(NodeMouseEventArgs evtArgs)
		{
			// TODO:  Add ConditionSysbol.OnMouseEnter implementation
			this.rect.FillStyle.Color = Color.BurlyWood;
			
		}
	
		protected override void OnMouseLeave(NodeMouseEventArgs evtArgs)
		{
			// TODO:  Add ConditionSysbol.OnMouseLeave implementation
			this.rect.FillStyle.Color = Color.Beige;
		}
		
	
		protected override void OnBoundsChanged(BoundsEventArgs evtArgs)
		{
			SetLabelToMiddle();
//			if (this.Labels.Count>0)
//			{
//				float fScaleX=evtArgs.NewBounds.X/evtArgs.OldBounds.X;
//				float fScaleY=evtArgs.NewBounds.Y/evtArgs.OldBounds.Y;
//				for (int i=this.Labels.Count-1; i>=0; i--)
//				{
//					this.Labels[i].Scale(fScaleX,fScaleY);
//				}
//			}
			
		}

		public override bool AcceptConnection(Port sourcePort, Port targetPort)
		{
			bool accept = false;

			// First, determine which port belongs to the foreign port container.

			Port foreignPort = null;

			if (this.Ports.Contains(sourcePort) || this.CenterPort == sourcePort)
			{
				foreignPort = targetPort;
			}
			else if (this.Ports.Contains(targetPort) || this.CenterPort == targetPort)
			{
				foreignPort = sourcePort;
			}

			if (foreignPort == null)
			{
				return false;  // This should never happen
			}

			// Now get the foreign port container and check to see if it is
			// a Link object. If so, then check to see if the foreign port
			// is the link's tail port or head port. If it's the tail port,
			// then always allow the connection, because a member symbol can
			// have any number of edges leaving it. If it's the head port,
			// then only allow the connection if the member symbol has no
			// other edges entering it yet.

			IPortContainer foreignContainer = foreignPort.Container;
			if (foreignContainer != null)
			{
				Link link = foreignContainer as Link;
				if (link != null)
				{
					if (foreignPort == link.TailPort)
					{
						MessageBox.Show("A result node cannot be a parent node.");
						accept = false;
					}
					else
					{
						IGraphNode curNode = this as IGraphNode;

						if (curNode != null)
						{
							ICollection edgesEntering = curNode.EdgesEntering;
							if (edgesEntering.Count == 0)
							{
								accept = true;
							}
							else
							{
								MessageBox.Show("A node can only have one parent.");
							}
						}
					}
				}
			}

			return accept;
		}

		


	}
}
