#region Copyright Syncfusion Inc. 2001 -2004
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
//  Copyright Syncfusion Inc. All rights reserved. Use of this code is subject to the terms of our 
//  license. A copy of the current license can be obtained at any time by e-mailing licensing@syncfusion.com. 
//  Re-distribution in any form is strictly prohibited. Any infringement will be prosecuted under applicable laws. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
#endregion 
 
using System;
using System.Collections;
using System.ComponentModel;

using Syncfusion.Windows.Forms.Diagram;

namespace Syncfusion.Windows.Forms.Diagram.Samples.OrgChart
{
	/// <summary>
	/// Summary description for OrgChartManager.
	/// </summary>
	public class clsLayoutManager : LayoutManager
	{
		public clsLayoutManager()
		{
		}

		public clsLayoutManager(Model mdl) : base(mdl)
		{
		}

		public override bool UpdateLayout(object contextInfo)
		{
			bool changesMade = true;
			
			if (this.Model != null)
			{
				ArrayList topRanking = new ArrayList();

				foreach (INode curNode in this.Model.Nodes)
				{
					tmpSymbol memberSym = curNode as tmpSymbol;
					if (memberSym != null)
					{
						if (memberSym.Rank == 1)
						{
							topRanking.Add(memberSym);
						}
						
					}
				}


				int numTopRanking = topRanking.Count;

				if (numTopRanking > 0)
				{
					float range = this.Model.Width / ((float) numTopRanking);
					float curLeft = this.Model.Bounds.Left;
					float curRight = curLeft + range;

					foreach (tmpSymbol curMember in topRanking)
					{
						this.PositionMember(curMember, curLeft, curRight);
						curLeft = curRight;
						curRight = curRight + range;
					}
				}
			}

			if (changesMade)
			{
				this.OnLayoutUpdated(new System.EventArgs());
			}

			return changesMade;
		}

		private void PositionMember(tmpSymbol memberSym, float left, float right)
		{
			int rank = memberSym.Rank;
			memberSym.Y = ((float) rank) * this.verticalSpacing;
			float symWidth = memberSym.Width;
			float rangeCenter = (left + right) / 2.0f;
			memberSym.X = rangeCenter - (symWidth / 2.0f);

			ICollection employees = memberSym.Employees;
			int numEmployees = employees.Count;

			if (numEmployees > 0)
			{
				float range = (right - left) / ((float) numEmployees);
				float curLeft = left;
				float curRight = curLeft + range;
				foreach (tmpSymbol employee in employees)
				{
					this.PositionMember(employee, curLeft, curRight);
					curLeft = curRight;
					curRight = curRight + range;
				}
			}
		}

		/// <summary>
		/// 
		/// </summary>
		[
			Browsable(true),
			DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)
		]
		public float HorizontalSpacing
		{
			get
			{
				return this.horizontalSpacing;
			}
			set
			{
				this.horizontalSpacing = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		[
		Browsable(true),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)
		]
		public float VerticalSpacing
		{
			get
			{
				return this.verticalSpacing;
			}
			set
			{
				this.verticalSpacing = value;
			}
		}

		private float horizontalSpacing = 200.0f;
		private float verticalSpacing = 140.0f;
	}
}
